/*

The originally measured seven initialization sequences from the S65 display can luckily be reduced to three sequences. In addition we do not need the very long time intervals Siemens used in here approach. The first (of the three) used initialization sequence is: 
INIT1: Display Software Reset 
 
0xFDFD, 0xFDFD

 
 It looks like this sequence starts the display controller. After a certain time (min 50ms) the second init command sequence is transmitted to the display. 
INIT2: 
 
0xEF00
0xEE04, 0x1B04
0xFEFE, 0xFEFE
0xEF90, 0x4A04, 0x7F3F, 0xEE04, 0x4306

 
After that we need to wait for the display internal voltage settling. This wait time is very sensitive. If the time is too short or too long the display will not work! It was measured to be 7ms and this value shall also be used to drive the display with the sequences shown here. 
 The second sequence follows directly and is: 
INIT3: 
 
0xEF90, 0x0983, 0x0800, 0x0BAF, 0x0A00, 0x0500, 0x0600, 0x0700
0xEF00
0xEE0C
0xEF90, 0x0080
0xEFB0, 0x4902
0xEF00
0x7F01, 0xE181
0xE202
0xE276
0xE183
0x8001

##############################################################
in der Library

; Init tables for S65-Display
INIT1:  ; 24bytes
//.byte 0xFD, 0xFD, 0xFD, 0xFD
.byte 0xEF, 0x00, 0xEE, 0x04, 0x1B, 0x04, 0xFE, 0xFE, 0xFE, 0xFE, 0xEF, 0x90, 0x4A, 0x04
.byte 0x7F, 0x3F, 0xEE, 0x04, 0x43, 0x06 

INIT2:  ; 42bytes
.byte 0xEF, 0x90,  0x09, 0x83,  0x08, 0x00,  0x0B, 0xAF,  0x0A, 0x00,  0x05, 0x00,  0x06, 0x00
.byte 0x07, 0x00,  0xEF, 0x00,  0xEE, 0x0C,  0xEF, 0x90,  0x00, 0x80,  0xEF, 0xB0,  0x49, 0x02 
.byte 0xEF, 0x00,  0x7F, 0x01,  0xE1, 0x81,  0xE2, 0x02,  0xE2, 0x76,  0xE1, 0x83,  0x80, 0x01



##############################################################
 
After the initialization sequence the display is ready to use. An immediate clear screen is recommended because the display shows random content after first power on.



                 15		    |         26
            p5.0 16	STE1	|         25
            p5.2 17	SOMI1	|         24
            p5.4 18	MCLK	|   SIMO1 23 p5.1 > DAT > 5
1 < RS    < p5.6 19	ACLK	|   UCLK1 22 p5.3 > CLK > 4
2 < RESET < p5.7 20	TBOUT	|   SMCLK 21 p5.5 > CS  > 3



 1 RS
 2 RESET
 3 CS
 4 CLK
 5 DAT
 6 2V9
 7 GND
 8 1V8
 9 LED+
10 LED GND

131
  |
  |
  |
  |
  |
  |
0_|______________________
  |
  0						175

*/
#include  <stdio.h>
#include  <io.h>
#include  <signal.h>
#include  <string.h>
#include "stdlib.h"

#include "lanimacros.h"
#include "display.h"
/*
#define DISP_RS		BIT6			// high = Command, low = Data
#define DISP_RESET	BIT7 			// aktiv low
#define DISP_CS		BIT5			// aktiv low
#define DISP_CLK	BIT3			// serial clock
#define DISP_DATA	BIT1			// serial data
*/

#define DISP_RS		BIT6			// high = Command, low = Data
#define DISP_RESET	BIT7 			// aktiv low
#define DISP_CS		BIT0			// aktiv low
#define DISP_CLK	BIT3			// serial clock
#define DISP_DATA	BIT1			// serial data


#include "font6x8.xbm"				//  static !const! unsigned char 
#include "font8x14.xbm"				//  static !const! unsigned char 
//#include "icons.h"

static volatile unsigned int hintergrundfarbe = COLOR_BLUE,zeichenfarbe= COLOR_RED;

static volatile unsigned int zeichenbreite = 6, zeichenhoehe = 8;

static unsigned char *zeichensatz = (unsigned char *)font6x8_bits;

static unsigned int cursor_x=1;
static unsigned int cursor_y=100;

static volatile unsigned char spibuffer=0,spiflag=0;


void initdisplay(void)
{
	unsigned int f;
	
	const unsigned int init_1[] = {	0xFDFD, 0xFDFD};
	
	const unsigned int init_2[] = {	0xEF00,
		0xEE04, 0x1B04,
		0xFEFE, 0xFEFE,
		0xEF90, 0x4A04, 0x7F3F, 0xEE04, 0x4306};
								
	const unsigned int init_3[] = {	0xEF90, 0x0983, 0x0800, 0x0BAF, 0x0A00, 0x0500, 0x0600, 0x0700,
			0xEF00,
			0xEE0C,
			0xEF90, 0x0080,
			0xEFB0, 0x4902,
			0xEF00,
			0x7F01, 0xE181,
			0xE202,
			0xE276,
			0xE183};
	const unsigned int init_4[] = {	0x8001,
			0xEF90,0x0000};


		setBit(P5DIR,DISP_CS|DISP_CLK|DISP_DATA);	//Ausgänge
		setBit(P4DIR,DISP_RS|DISP_RESET);
		//clearBit(P5OUT,DISP_DATA);										//select display
		setBit(P5OUT,DISP_CLK|DISP_CS);									//deselect display

		/*
		Note: Initializing or Re-Configuring the USART Module
		The required USART initialization/re-configuration process is:
		1) Set SWRST (BIS.B #SWRST,&UxCTL)
		2) Initialize all USART registers with SWRST=1 (including UxCTL)
		3) Enable USART module via the MEx SFRs (USPIEx)
		4) Clear SWRST via software (BIC.B         #SWRST,&UxCTL)
		5) Enable interrupts (optional) via the IEx SFRs (URXIEx and/or UTXIEx)
		Failure to follow this process may result in unpredictable USART

		*/
		//###################################################

		UCTL1 = SWRST;

		UTCTL1 = SSEL1+SSEL0+STC+CKPH;        // SMCLK, 3-pin mode
		setBit(UCTL1,CHAR|SYNC|MM);           // 8-bit SPI Master **SWRST**
		UBR01 = 0x02;                         // UCLK/2 
		UBR11 = 0x00;                         // 0
		UMCTL1 = 0x00;                        // no modulation
		P5SEL |= DISP_CLK|DISP_DATA;          // P3.1-3 SPI option select

		ME2 |= USPIE1;                        // Enable USART1 SPI mode
	

		clearBit(UCTL1,SWRST);
		IE2   |=  UTXIE1;              // Enable USART1 TX interrupt
		//####################################################

		clearBit(P4OUT,DISP_RESET);										//reset Display

		timerdelay(1000);

		setBit(P4OUT,DISP_RESET);


		eint();
		
		displaycommand(init_1[0]);
		displaycommand(init_1[1]);

		timerdelay(50000);

		for(f=0;f<=9;f++)
		{
			displaycommand(init_2[f]);
		}

		timerdelay(7000);

		for(f=0;f<=19;f++)
		{
			displaycommand(init_3[f]);
		}
		
		timerdelay(50000);
		
		for(f=0;f<=2;f++)
		{
			displaycommand(init_4[f]);
		}
		
		
	
		displaysetfont(0);
		displaycls(COLOR_BLACK);
	
}

void displaydata(unsigned int befehl)
{
	//unsigned int f,test=0x8000;
	
	clearBit(P4OUT,DISP_RS);								//Daten	
	clearBit(P5OUT,DISP_CS);								//select Display

	while (spiflag!=0)
	{
		nop();
	}
	spiflag = 2;
	spibuffer = (unsigned char)(befehl);    			// zweiter Teil wird im Interrupt übertragen
	TXBUF1 = (unsigned char)(befehl>>8);                      
	
}

void displaycommand(unsigned int befehl)
{
	//unsigned int f,test=0x8000;
	
	setBit(P4OUT,DISP_RS);								//Befehl	
	clearBit(P5OUT,DISP_CS);							//select Display
	
	while (spiflag!=0)
	{
		nop();
	}
	spiflag = 2;
	spibuffer = (unsigned char)(befehl);   				// zweiter Teil wird im Interrupt übertragen
	TXBUF1 = (unsigned char)(befehl>>8);                      
}


void display_setcursor(unsigned int x, unsigned int y)
{
	cursor_x=x;
	cursor_y=y;
}

int display_putc(int c)
{
	
	
	switch ((char)c)
	{
		case  '\n':          
			cursor_y=cursor_y-zeichenhoehe-2;	
			break;
		case  '\r':
			cursor_x=1;              
			break;                              
		case  '\t': 		
			cursor_x=cursor_x+(zeichenbreite*4);		
			break;
			default:				// n unkomprimierte daten
				write(cursor_x,cursor_y,(char)c);
				cursor_x=cursor_x+zeichenbreite;	
								
	}
	
	
	
	
	if(cursor_x>(175-zeichenbreite))
	{
		cursor_x=1;
		cursor_y=cursor_y-zeichenhoehe-2;
	}
	
	if(cursor_y>(131-zeichenhoehe))
	{	
		cursor_y=131-zeichenhoehe;
	}
	
	return 0;
}

void displaycolor(unsigned int farbe1, unsigned int farbe2)
{
	zeichenfarbe= farbe1;
	hintergrundfarbe = farbe2;
}

unsigned int color(unsigned int r, unsigned int g, unsigned int b)
{
	return((r<<11)|(g<<6)|b);
}


void displaysetfont(unsigned int nummer)
{
	if(nummer==0)
	{
		zeichenbreite = 6; 
		zeichenhoehe = 8;

		zeichensatz = (unsigned char *)font6x8_bits;
	}else
	{
		zeichenbreite = 8; 
		zeichenhoehe = 14;

		zeichensatz = (unsigned char *)font8x14_bits;
	
	}
	
	cursor_x=1;
	cursor_y=131-zeichenhoehe;
	
}
	
void writestring(unsigned int x, unsigned int y, unsigned char *text)
{
	unsigned int f=0;
	
	while(text[f]!= 0)
	{
		if(text[f]>=32)
		{	
			write(x,y,text[f]);
			//x=x+8;
			x=x+zeichenbreite;
		}
			f++;
	}
	
}

/*
void displayicon(unsigned int x1, unsigned int y1, unsigned int nummer)
{
	unsigned int f,n,zaehler=0;
	
	unsigned int x2 = 32 + x1 - 1; 
	unsigned int y2 = 32 + y1 - 1;
	
	
	displaycommand(0x0504);
	displaycommand(0x0800+y1);
	displaycommand(0x0900+y2);
	displaycommand(0x0A00+x1);
	displaycommand(0x0B00+x2);
	
	zaehler=nummer*(32*32);
	
	for(f=x1;f<=x2;f++)
	{
		for(n=y1;n<=y2;n++)
		{
			displaydata(icons_farbtabelle[icons_daten[zaehler++]]);
		}
	}
}
*/
void displaypic(unsigned int x1, unsigned int y1, 
				unsigned int x2, unsigned int y2,
				unsigned int kompression,
				unsigned int *farbtabelle, unsigned char *daten)
{
	unsigned int f,n,zaehler=0;
	
	x2 = x2 + x1 - 1; 
	y2 = y2 + y1 - 1;
	
	unsigned int wert1,wert2;
	
	displaycommand(0x0504);
	displaycommand(0x0800+y1);
	displaycommand(0x0900+y2);
	displaycommand(0x0A00+x1);
	displaycommand(0x0B00+x2);
	

	if (kompression == 0)
	{
		for(f=x1;f<=x2;f++)
		{
			for(n=y1;n<=y2;n++)
			{
				displaydata(farbtabelle[daten[zaehler++]]);
			}
		}
	}
		/*
		Jeweils zwei aufeinanderfolgende Bytes bilden einen Datensatz. 
		Hat das erste Byte einen anderen Wert als 0, so wird das zweite Byte so oft (bei BI_RLE4: halb so oft) wiederholt, 
		wie das erste Byte angibt. Hat das erste Byte hingegen den Wert 0, so hängt die Bedeutung vom zweiten Byte ab: 
		
		0:  Ende der Bildzeile.
		1:  Ende der Bitmap.
		2:  Verschiebung der aktuellen Pixelposition. Die beiden nächsten Bytes geben die Verschiebung nach rechts und nach unten an.
		n=3-255:  Die folgenden n Bytes (bei BI_RLE4: die folgenden n Nibbles) werden direkt übernommen; der nächste Datensatz findet
	 	sich am darauffolgenden geraden Offset (vom Start der Bilddaten aus gezählt).
		Das Resultat wird wie im unkomprimierten Fall interpretiert.
		*/
	else
	{
		do
		{
			
			wert1= daten[zaehler++];
			wert2= daten[zaehler++];
			
			if(wert1 !=0)
			{
				for(f=0;f<wert1;f++)
				{
					displaydata(farbtabelle[wert2]);
				}
			}
			else
			{
				switch (wert2)
				{
					case  0:                // Ende der Zeile
						nop();				// wird durch das Clipping nicht gebraucht
						break;
					case  1:                // Ende der Bitmap
						return;				// Funktion beenden
						break;                              
					case  2: 			 	// Verschiebung der aktuellen Pixelposition...
							nop();				// wird nicht unterstuetzt...
							break;
					default:				// n unkomprimierte daten
					
						for(f=0;f<wert2;f++)
						{
							displaydata(farbtabelle[daten[zaehler++]]);
						}
						if((wert2 % 2) !=0)
						{
							zaehler++;
						}
								
				}
			}
		}while(1); //while(zaehler<test_biSizeImage);
	}
}

void displaybox(unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2, unsigned int farbe)
{
	unsigned int f,n;
	
	displaycommand(0x0504);
	displaycommand(0x0800+y1);
	displaycommand(0x0900+y2);
	displaycommand(0x0A00+x1);
	displaycommand(0x0B00+x2);
	
	for(f=x1;f<=x2;f++)
	{
		for(n=y1;n<=y2;n++)
		{
				displaydata(farbe);
		}
	}
	
}

void write(unsigned int x, unsigned int y, char c)
{
	unsigned int f,n;
	unsigned char test;
	//font_bits[]

	displaycommand(0xEF90);

	
	displaycommand(0x0504);
	displaycommand(0x0800+y);
	displaycommand(0x0900+y+(zeichenhoehe-1));
	displaycommand(0x0A00+x);
	displaycommand(0x0B00+x+(zeichenbreite-1));

	for(f=zeichenhoehe;f>0;f--)
	{
		test=1;
		for(n=0;n<zeichenbreite;n++)
		{
			//if((zeichensatz[c-32][f-1] & test)==test)
			if((zeichensatz[( (c-32) * zeichenhoehe ) + (f-1) ] & test)==test)
			{
				displaydata(zeichenfarbe);
			}
			else
			{
				displaydata(hintergrundfarbe);
			}
			test=test*2;
		}
	}
}

void displayrgb(unsigned int r, unsigned int g, unsigned int b)
{
	unsigned int farbe;
	
	r = (r & 0x1F)<<11;
	g = (g & 0x3F)<<5;
	b = (b & 0x1F);
	
	farbe= r|g|b;
	
	displaycls(farbe);
}

void displayplot(unsigned int x, unsigned int y, unsigned int farbe)
{
	
	displaycommand(0xEF90);
	displaycommand(0x0504);
	displaycommand(0x0600 | y);
	displaycommand(0x0700 | x);
	
	displaydata(farbe);
}

void displayline(uint16_t x1, uint16_t y1, uint16_t x2, uint16_t y2, unsigned int farbe)
{
	int16_t  dx = x2 - x1;
	int16_t   dy = y2 - y1;
	uint16_t ax = abs(dx) << 1;
	uint16_t ay = abs(dy) << 1;
	int16_t   sx = (dx >= 0) ? 1 : -1;
	int16_t   sy = (dy >= 0) ? 1 : -1;

	uint16_t x = x1;
	uint16_t y = y1;

	if (ax > ay) {
		int16_t d = ay - (ax >> 1);
		while (x != x2) {
			displayplot(x, y, farbe);
			if (d > 0 || (d == 0 && sx == 1)) {
				y += sy;
				d -= ax;
			}
			x += sx;
			d += ay;
		}
	} else {
		int16_t d = ax - (ay >> 1);
		while (y != y2) {
			displayplot(x, y, farbe);
			if (d > 0 || (d == 0 && sy == 1)) {
				x += sx;
				d -= ay;
			}
			y += sy;
			d += ax;
		}
	}
	displayplot(x, y, farbe);
}

void displayset(unsigned int x, unsigned int y)
{
	
	displaycommand(0xEF90);
	displaycommand(0x0504);
	displaycommand(0x0600 | y);
	displaycommand(0x0700 | x);
}

void displaycls(unsigned int farbe)
{
	unsigned int f;
	
	displaycommand(0xEF90);

	
	displaycommand(0x0504);
	displaycommand(0x0800+0);
	displaycommand(0x0900+131);
	displaycommand(0x0A00+0);
	displaycommand(0x0B00+175);
	
	for(f=0;f<0x5AC0;f++)
	{
		displaydata(farbe);
	}
	
	cursor_x=1;
	cursor_y=131-zeichenhoehe;
	
}

void timerdelay(unsigned int zeit)
{
	
	TACTL = ID0 + ID1 + TASSEL1 + TACLR;       		// SMCLK / 8 , clear TAR
	TACTL |= MC1;                         			// Start Timer_A in continuous mode
	while(TAR<zeit){};
	
	
	TACTL = ID0 + ID1 + TASSEL1 + TACLR; 			// Stop Timer
}

interrupt (UART1TX_VECTOR) spi_interrupt(void)
{
	if(spiflag==2)		
	{
		TXBUF1 = spibuffer;     // zweites Byte uebertragen
		spiflag=1;				// Flag = 1. Interrupt
	
	}else
	{
		//timerdelay(1);
		setBit(P5OUT,DISP_CS);	// Display Chip Select
		spiflag=0;				// SPI freigeben
	}
}

